@@ -3,10 +3,11 @@ import typing as tp
 import jax
 import jax.numpy as jnp
 import numpy as np
+import treeo as to
 from flax.linen import normalization as flax_module
 
-from treex import types
-from treex.module import Module
+from treex import types, utils
+from treex.module import Module, next_key
 
 
 class BatchNorm(Module):
@@ -23,15 +24,14 @@ class BatchNorm(Module):
     """
 
     # pytree
-    mean: types.BatchStat[jnp.ndarray, None]
-    var: types.BatchStat[jnp.ndarray, None]
-    scale: types.Parameter[jnp.ndarray, None]
-    bias: types.Parameter[jnp.ndarray, None]
-    momentum: types.DiffHyperParam[jnp.ndarray]
+    mean: tp.Optional[jnp.ndarray] = types.BatchStat.node()
+    var: tp.Optional[jnp.ndarray] = types.BatchStat.node()
+    scale: tp.Optional[jnp.ndarray] = types.Parameter.node()
+    bias: tp.Optional[jnp.ndarray] = types.Parameter.node()
+    momentum: jnp.ndarray = to.node()
 
     # props
-    features_in: int
-    axis: int = -1
+    axis: int
     epsilon: float
     dtype: flax_module.Dtype
     use_bias: bool
@@ -49,7 +49,7 @@ class BatchNorm(Module):
 
     def __init__(
         self,
-        features_in: int,
+        *,
         axis: int = -1,
         momentum: tp.Union[float, jnp.ndarray] = 0.99,
         epsilon: float = 1e-5,
@@ -70,8 +70,6 @@ class BatchNorm(Module):
         """
         Arguments:
             features_in: the number of input features.
-            use_running_average: if True, the statistics stored in batch_stats
-                will be used instead of computing the batch statistics on the input.
             axis: the feature or non-batch axis of the input.
             momentum: decay rate for the exponential moving average of
                 the batch statistics.
@@ -91,8 +89,7 @@ class BatchNorm(Module):
                 the examples on the first two and last two devices. See `jax.lax.psum`
                 for more details.
         """
-        super().__init__()
-        self.features_in = features_in
+
         self.axis = axis
         self.momentum = jnp.asarray(momentum)
         self.epsilon = epsilon
@@ -125,32 +122,8 @@ class BatchNorm(Module):
             axis_index_groups=self.axis_index_groups,
         )
 
-    def module_init(self, key: jnp.ndarray):
-
-        batch_size = 10  # random
-
-        shape = [batch_size] * (abs(self.module.axis) + 1)  # overcompensate
-        shape[self.module.axis] = self.features_in
-
-        x = jax.random.uniform(key, shape=shape)
-
-        variables = self.module.init(key, x, use_running_average=True).unfreeze()
-
-        # Extract collections
-        if "params" in variables:
-            params = variables["params"]
-
-            if self.use_bias:
-                self.bias = params["bias"]
-
-            if self.use_scale:
-                self.scale = params["scale"]
-
-        self.mean = variables["batch_stats"]["mean"]
-        self.var = variables["batch_stats"]["var"]
-
     def __call__(
-        self, x: np.ndarray, use_running_average: tp.Optional[bool] = None
+        self, x: jnp.ndarray, use_running_average: tp.Optional[bool] = None
     ) -> jnp.ndarray:
         """Normalizes the input using batch statistics.
 
@@ -162,7 +135,25 @@ class BatchNorm(Module):
         Returns:
             Normalized inputs (the same shape as inputs).
         """
-        assert self.initialized, "Module not initialized"
+        if self.initializing():
+            variables = self.module.init(
+                next_key(),
+                x,
+                use_running_average=True,
+            ).unfreeze()
+
+            # Extract collections
+            if "params" in variables:
+                params = variables["params"]
+
+                if self.use_bias:
+                    self.bias = params["bias"]
+
+                if self.use_scale:
+                    self.scale = params["scale"]
+
+            self.mean = variables["batch_stats"]["mean"]
+            self.var = variables["batch_stats"]["var"]
 
         params = {}
 
@@ -184,7 +175,7 @@ class BatchNorm(Module):
         training = (
             not use_running_average
             if use_running_average is not None
-            else self.training and not self.frozen
+            else self.training and not self.frozen and self.initialized
         )
 
         # call apply
